@@ -8,7 +8,7 @@ int print_addrinfo(const struct sockaddr_nl *who,
 int print_addrlabel(const struct sockaddr_nl *who,
 		    struct nlmsghdr *n, void *arg);
 int print_neigh(const struct sockaddr_nl *who,
-	        struct nlmsghdr *n, void *arg);
+		struct nlmsghdr *n, void *arg);
 int ipaddr_list_link(int argc, char **argv);
 void ipaddr_get_vf_rate(int, int *, int *, int);
 void iplink_usage(void) __attribute__((noreturn));
@@ -43,6 +43,7 @@ int do_iptunnel(int argc, char **argv);
 int do_ip6tunnel(int argc, char **argv);
 int do_iptuntap(int argc, char **argv);
 int do_iplink(int argc, char **argv);
+int do_ipmacsec(int argc, char **argv);
 int do_ipmonitor(int argc, char **argv);
 int do_multiaddr(int argc, char **argv);
 int do_multiroute(int argc, char **argv);
@@ -51,6 +52,7 @@ int do_netns(int argc, char **argv);
 int do_xfrm(int argc, char **argv);
 int do_ipl2tp(int argc, char **argv);
 int do_ipfou(int argc, char **argv);
+extern int do_ipila(int argc, char **argv);
 int do_tcp_metrics(int argc, char **argv);
 int do_ipnetconf(int argc, char **argv);
 int do_iptoken(int argc, char **argv);
@@ -59,6 +61,7 @@ int iplink_get(unsigned int flags, char *name, __u32 filt_mask);
 static inline int rtm_get_table(struct rtmsg *r, struct rtattr **tb)
 {
 	__u32 table = r->rtm_table;
+
 	if (tb[RTA_TABLE])
 		table = rta_getattr_u32(tb[RTA_TABLE]);
 	return table;
@@ -68,8 +71,7 @@ extern struct rtnl_handle rth;
 
 #include <stdbool.h>
 
-struct link_util
-{
+struct link_util {
 	struct link_util	*next;
 	const char		*id;
 	int			maxattr;
@@ -81,14 +83,15 @@ struct link_util
 					     struct rtattr *);
 	void			(*print_help)(struct link_util *, int, char **,
 					     FILE *);
-	bool			slave;
 };
 
 struct link_util *get_link_kind(const char *kind);
-struct link_util *get_link_slave_kind(const char *slave_kind);
 
 void br_dump_bridge_id(const struct ifla_bridge_id *id, char *buf, size_t len);
 
+__u32 ipvrf_get_table(const char *name);
+bool name_is_vrf(const char *name);
+
 #ifndef	INFINITY_LIFE_TIME
 #define     INFINITY_LIFE_TIME      0xFFFFFFFFU
 #endif
