@@ -29,20 +29,22 @@
 #include <getopt.h>
 
 #include <libnetlink.h>
+#include <json_writer.h>
 #include <linux/if.h>
 #include <linux/if_link.h>
 
 #include <SNAPSHOT.h>
 
-int dump_zeros = 0;
-int reset_history = 0;
-int ignore_history = 0;
-int no_output = 0;
-int json_output = 0;
-int no_update = 0;
-int scan_interval = 0;
-int time_constant = 0;
-int show_errors = 0;
+int dump_zeros;
+int reset_history;
+int ignore_history;
+int no_output;
+int json_output;
+int no_update;
+int scan_interval;
+int time_constant;
+int show_errors;
+int pretty;
 double W;
 char **patterns;
 int npatterns;
@@ -52,8 +54,7 @@ int source_mismatch;
 
 #define MAXS (sizeof(struct rtnl_link_stats)/sizeof(__u32))
 
-struct ifstat_ent
-{
+struct ifstat_ent {
 	struct ifstat_ent	*next;
 	char			*name;
 	int			ifindex;
@@ -98,7 +99,7 @@ static int match(const char *id)
 	if (npatterns == 0)
 		return 1;
 
-	for (i=0; i<npatterns; i++) {
+	for (i = 0; i < npatterns; i++) {
 		if (!fnmatch(patterns[i], id, 0))
 			return 1;
 	}
@@ -109,7 +110,7 @@ static int get_nlmsg(const struct sockaddr_nl *who,
 		     struct nlmsghdr *m, void *arg)
 {
 	struct ifinfomsg *ifi = NLMSG_DATA(m);
-	struct rtattr * tb[IFLA_MAX+1];
+	struct rtattr *tb[IFLA_MAX+1];
 	int len = m->nlmsg_len;
 	struct ifstat_ent *n;
 	int i;
@@ -135,7 +136,7 @@ static int get_nlmsg(const struct sockaddr_nl *who,
 	n->name = strdup(RTA_DATA(tb[IFLA_IFNAME]));
 	memcpy(&n->ival, RTA_DATA(tb[IFLA_STATS]), sizeof(n->ival));
 	memset(&n->rate, 0, sizeof(n->rate));
-	for (i=0; i<MAXS; i++)
+	for (i = 0; i < MAXS; i++)
 		n->val[i] = n->ival[i];
 	n->next = kern_db;
 	kern_db = n;
@@ -207,8 +208,9 @@ static void load_raw_table(FILE *fp)
 		n->name = strdup(p);
 		p = next;
 
-		for (i=0; i<MAXS; i++) {
-			unsigned rate;
+		for (i = 0; i < MAXS; i++) {
+			unsigned int rate;
+
 			if (!(next = strchr(p, ' ')))
 				abort();
 			*next++ = 0;
@@ -238,21 +240,26 @@ static void load_raw_table(FILE *fp)
 
 static void dump_raw_db(FILE *fp, int to_hist)
 {
+	json_writer_t *jw = json_output ? jsonw_new(fp) : NULL;
 	struct ifstat_ent *n, *h;
-	const char *eol = "\n";
 
 	h = hist_db;
-	if (json_output)
-		fprintf(fp, "{ \"%s\":{", info_source);
-	else
+	if (jw) {
+		jsonw_start_object(jw);
+		jsonw_pretty(jw, pretty);
+		jsonw_name(jw, info_source);
+		jsonw_start_object(jw);
+	} else
 		fprintf(fp, "#%s\n", info_source);
 
-	for (n=kern_db; n; n=n->next) {
+	for (n = kern_db; n; n = n->next) {
 		int i;
 		unsigned long long *vals = n->val;
 		double *rates = n->rate;
+
 		if (!match(n->name)) {
 			struct ifstat_ent *h1;
+
 			if (!to_hist)
 				continue;
 			for (h1 = h; h1; h1 = h1->next) {
@@ -265,22 +272,27 @@ static void dump_raw_db(FILE *fp, int to_hist)
 			}
 		}
 
-		if (json_output) {
-			fprintf(fp, "%s   \"%s\":{",
-				eol, n->name);
-			eol = ",\n";
-			for (i=0; i<MAXS && stats[i]; i++)
-				fprintf(fp, " \"%s\":%llu",
-					stats[i], vals[i]);
-			fprintf(fp, "}");
+		if (jw) {
+			jsonw_name(jw, n->name);
+			jsonw_start_object(jw);
+
+			for (i = 0; i < MAXS && stats[i]; i++)
+				jsonw_uint_field(jw, stats[i], vals[i]);
+			jsonw_end_object(jw);
 		} else {
 			fprintf(fp, "%d %s ", n->ifindex, n->name);
-			for (i=0; i<MAXS; i++)
+			for (i = 0; i < MAXS; i++)
 				fprintf(fp, "%llu %u ", vals[i],
-					(unsigned)rates[i]);
+					(unsigned int)rates[i]);
 			fprintf(fp, "\n");
 		}
 	}
+	if (jw) {
+		jsonw_end_object(jw);
+
+		jsonw_end_object(jw);
+		jsonw_destroy(&jw);
+	}
 }
 
 /* use communication definitions of meg/kilo etc */
@@ -301,18 +313,19 @@ static void format_rate(FILE *fp, const unsigned long long *vals,
 		fprintf(fp, "%8llu ", vals[i]);
 
 	if (rates[i] > mega) {
-		sprintf(temp, "%uM", (unsigned)(rates[i]/mega));
+		sprintf(temp, "%uM", (unsigned int)(rates[i]/mega));
 		fprintf(fp, "%-6s ", temp);
 	} else if (rates[i] > kilo) {
-		sprintf(temp, "%uK", (unsigned)(rates[i]/kilo));
+		sprintf(temp, "%uK", (unsigned int)(rates[i]/kilo));
 		fprintf(fp, "%-6s ", temp);
 	} else
-		fprintf(fp, "%-6u ", (unsigned)rates[i]);
+		fprintf(fp, "%-6u ", (unsigned int)rates[i]);
 }
 
 static void format_pair(FILE *fp, const unsigned long long *vals, int i, int k)
 {
 	char temp[64];
+
 	if (vals[i] > giga)
 		fprintf(fp, "%7lluM ", vals[i]/mega);
 	else if (vals[i] > mega)
@@ -321,13 +334,13 @@ static void format_pair(FILE *fp, const unsigned long long *vals, int i, int k)
 		fprintf(fp, "%8llu ", vals[i]);
 
 	if (vals[k] > giga) {
-		sprintf(temp, "%uM", (unsigned)(vals[k]/mega));
+		sprintf(temp, "%uM", (unsigned int)(vals[k]/mega));
 		fprintf(fp, "%-6s ", temp);
 	} else if (vals[k] > mega) {
-		sprintf(temp, "%uK", (unsigned)(vals[k]/kilo));
+		sprintf(temp, "%uK", (unsigned int)(vals[k]/kilo));
 		fprintf(fp, "%-6s ", temp);
 	} else
-		fprintf(fp, "%-6u ", (unsigned)vals[k]);
+		fprintf(fp, "%-6u ", (unsigned int)vals[k]);
 }
 
 static void print_head(FILE *fp)
@@ -338,55 +351,53 @@ static void print_head(FILE *fp)
 	fprintf(fp, "%8s/%-6s ", "RX Pkts", "Rate");
 	fprintf(fp, "%8s/%-6s ", "TX Pkts", "Rate");
 	fprintf(fp, "%8s/%-6s ", "RX Data", "Rate");
-	fprintf(fp, "%8s/%-6s\n","TX Data", "Rate");
+	fprintf(fp, "%8s/%-6s\n", "TX Data", "Rate");
 
 	if (!show_errors) {
 		fprintf(fp, "%-15s ", "");
 		fprintf(fp, "%8s/%-6s ", "RX Errs", "Drop");
 		fprintf(fp, "%8s/%-6s ", "TX Errs", "Drop");
 		fprintf(fp, "%8s/%-6s ", "RX Over", "Rate");
-		fprintf(fp, "%8s/%-6s\n","TX Coll", "Rate");
+		fprintf(fp, "%8s/%-6s\n", "TX Coll", "Rate");
 	} else {
 		fprintf(fp, "%-15s ", "");
 		fprintf(fp, "%8s/%-6s ", "RX Errs", "Rate");
 		fprintf(fp, "%8s/%-6s ", "RX Drop", "Rate");
 		fprintf(fp, "%8s/%-6s ", "RX Over", "Rate");
-		fprintf(fp, "%8s/%-6s\n","RX Leng", "Rate");
+		fprintf(fp, "%8s/%-6s\n", "RX Leng", "Rate");
 
 		fprintf(fp, "%-15s ", "");
 		fprintf(fp, "%8s/%-6s ", "RX Crc", "Rate");
 		fprintf(fp, "%8s/%-6s ", "RX Frm", "Rate");
 		fprintf(fp, "%8s/%-6s ", "RX Fifo", "Rate");
-		fprintf(fp, "%8s/%-6s\n","RX Miss", "Rate");
+		fprintf(fp, "%8s/%-6s\n", "RX Miss", "Rate");
 
 		fprintf(fp, "%-15s ", "");
 		fprintf(fp, "%8s/%-6s ", "TX Errs", "Rate");
 		fprintf(fp, "%8s/%-6s ", "TX Drop", "Rate");
 		fprintf(fp, "%8s/%-6s ", "TX Coll", "Rate");
-		fprintf(fp, "%8s/%-6s\n","TX Carr", "Rate");
+		fprintf(fp, "%8s/%-6s\n", "TX Carr", "Rate");
 
 		fprintf(fp, "%-15s ", "");
 		fprintf(fp, "%8s/%-6s ", "TX Abrt", "Rate");
 		fprintf(fp, "%8s/%-6s ", "TX Fifo", "Rate");
 		fprintf(fp, "%8s/%-6s ", "TX Hear", "Rate");
-		fprintf(fp, "%8s/%-6s\n","TX Wind", "Rate");
+		fprintf(fp, "%8s/%-6s\n", "TX Wind", "Rate");
 	}
 }
 
-static void print_one_json(FILE *fp, const struct ifstat_ent *n,
+static void print_one_json(json_writer_t *jw, const struct ifstat_ent *n,
 			   const unsigned long long *vals)
 {
-	int i, m;
-	const char *sep = " ";
+	int i, m = show_errors ? 20 : 10;
 
-	m = show_errors ? 20 : 10;
-	fprintf(fp, "    \"%s\":{", n->name);
-	for (i=0; i < m && stats[i]; i++) {
-		fprintf(fp, "%s\"%s\":%llu",
-			sep, stats[i], vals[i]);
-		sep = ", ";
-	}
-	fprintf(fp, " }");
+	jsonw_name(jw, n->name);
+	jsonw_start_object(jw);
+
+	for (i = 0; i < m && stats[i]; i++)
+		jsonw_uint_field(jw, stats[i], vals[i]);
+
+	jsonw_end_object(jw);
 }
 
 static void print_one_if(FILE *fp, const struct ifstat_ent *n,
@@ -395,7 +406,7 @@ static void print_one_if(FILE *fp, const struct ifstat_ent *n,
 	int i;
 
 	fprintf(fp, "%-15s ", n->name);
-	for (i=0; i<4; i++)
+	for (i = 0; i < 4; i++)
 		format_rate(fp, vals, n->rate, i);
 	fprintf(fp, "\n");
 
@@ -439,42 +450,45 @@ static void print_one_if(FILE *fp, const struct ifstat_ent *n,
 
 static void dump_kern_db(FILE *fp)
 {
+	json_writer_t *jw = json_output ? jsonw_new(fp) : NULL;
 	struct ifstat_ent *n;
-	const char *eol = "\n";
 
-	if (json_output)
-		fprintf(fp, "{ \"%s\": {", info_source);
-	else
+	if (jw) {
+		jsonw_start_object(jw);
+		jsonw_pretty(jw, pretty);
+		jsonw_name(jw, info_source);
+		jsonw_start_object(jw);
+	} else
 		print_head(fp);
 
-	for (n=kern_db; n; n=n->next) {
+	for (n = kern_db; n; n = n->next) {
 		if (!match(n->name))
 			continue;
 
-		if (json_output) {
-			fprintf(fp, "%s", eol);
-			eol = ",\n";
-			print_one_json(fp, n, n->val);
-		} else
+		if (jw)
+			print_one_json(jw, n, n->val);
+		else
 			print_one_if(fp, n, n->val);
 	}
 	if (json_output)
 		fprintf(fp, "\n} }\n");
 }
 
-
 static void dump_incr_db(FILE *fp)
 {
 	struct ifstat_ent *n, *h;
-	const char *eol = "\n";
+	json_writer_t *jw = json_output ? jsonw_new(fp) : NULL;
 
 	h = hist_db;
-	if (json_output)
-		fprintf(fp, "{ \"%s\":{", info_source);
-	else
+	if (jw) {
+		jsonw_start_object(jw);
+		jsonw_pretty(jw, pretty);
+		jsonw_name(jw, info_source);
+		jsonw_start_object(jw);
+	} else
 		print_head(fp);
 
-	for (n=kern_db; n; n=n->next) {
+	for (n = kern_db; n; n = n->next) {
 		int i;
 		unsigned long long vals[MAXS];
 		struct ifstat_ent *h1;
@@ -492,17 +506,19 @@ static void dump_incr_db(FILE *fp)
 		if (!match(n->name))
 			continue;
 
-		if (json_output) {
-			fprintf(fp, "%s", eol);
-			eol = ",\n";
-			print_one_json(fp, n, n->val);
-		} else
+		if (jw)
+			print_one_json(jw, n, n->val);
+		else
 			print_one_if(fp, n, vals);
 	}
-	if (json_output)
-		fprintf(fp, "\n} }\n");
-}
 
+	if (jw) {
+		jsonw_end_object(jw);
+
+		jsonw_end_object(jw);
+		jsonw_destroy(&jw);
+	}
+}
 
 static int children;
 
@@ -524,9 +540,11 @@ static void update_db(int interval)
 
 	for (n = kern_db; n; n = n->next) {
 		struct ifstat_ent *h1;
+
 		for (h1 = h; h1; h1 = h1->next) {
 			if (h1->ifindex == n->ifindex) {
 				int i;
+
 				for (i = 0; i < MAXS; i++) {
 					if ((long)(h1->ival[i] - n->ival[i]) < 0) {
 						memset(n->ival, 0, sizeof(n->ival));
@@ -536,6 +554,7 @@ static void update_db(int interval)
 				for (i = 0; i < MAXS; i++) {
 					double sample;
 					unsigned long incr = h1->ival[i] - n->ival[i];
+
 					n->val[i] += incr;
 					n->ival[i] = h1->ival[i];
 					sample = (double)(incr*1000)/interval;
@@ -546,6 +565,7 @@ static void update_db(int interval)
 							n->rate[i] = sample;
 						} else {
 							double w = W*(double)interval/scan_interval;
+
 							n->rate[i] += w*(sample-n->rate[i]);
 						}
 					}
@@ -553,6 +573,7 @@ static void update_db(int interval)
 
 				while (h != h1) {
 					struct ifstat_ent *tmp = h;
+
 					h = h->next;
 					free(tmp->name);
 					free(tmp);
@@ -566,13 +587,14 @@ static void update_db(int interval)
 	}
 }
 
-#define T_DIFF(a,b) (((a).tv_sec-(b).tv_sec)*1000 + ((a).tv_usec-(b).tv_usec)/1000)
+#define T_DIFF(a, b) (((a).tv_sec-(b).tv_sec)*1000 + ((a).tv_usec-(b).tv_usec)/1000)
 
 
 static void server_loop(int fd)
 {
 	struct timeval snaptime = { 0 };
 	struct pollfd p;
+
 	p.fd = fd;
 	p.events = p.revents = POLLIN;
 
@@ -583,7 +605,7 @@ static void server_loop(int fd)
 
 	for (;;) {
 		int status;
-		int tdiff;
+		time_t tdiff;
 		struct timeval now;
 
 		gettimeofday(&now, NULL);
@@ -594,24 +616,24 @@ static void server_loop(int fd)
 			tdiff = 0;
 		}
 
-		if (poll(&p, 1, tdiff + scan_interval) > 0
+		if (poll(&p, 1, scan_interval - tdiff) > 0
 		    && (p.revents&POLLIN)) {
 			int clnt = accept(fd, NULL, NULL);
+
 			if (clnt >= 0) {
 				pid_t pid;
+
 				if (children >= 5) {
 					close(clnt);
 				} else if ((pid = fork()) != 0) {
-					if (pid>0)
+					if (pid > 0)
 						children++;
 					close(clnt);
 				} else {
 					FILE *fp = fdopen(clnt, "w");
-					if (fp) {
-						if (tdiff > 0)
-							update_db(tdiff);
+
+					if (fp)
 						dump_raw_db(fp, 0);
-					}
 					exit(0);
 				}
 			}
@@ -626,7 +648,7 @@ static int verify_forging(int fd)
 	struct ucred cred;
 	socklen_t olen = sizeof(cred);
 
-	if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, (void*)&cred, &olen) ||
+	if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, (void *)&cred, &olen) ||
 	    olen < sizeof(cred))
 		return -1;
 	if (cred.uid == getuid() || cred.uid == 0)
@@ -646,6 +668,7 @@ static void usage(void)
 "   -e, --errors	show errors\n"
 "   -j, --json          format output in JSON\n"
 "   -n, --nooutput	do history only\n"
+"   -p, --pretty        pretty print\n"
 "   -r, --reset		reset history\n"
 "   -s, --noupdate	don\'t update history\n"
 "   -t, --interval=SECS	report average over the last SECS\n"
@@ -663,6 +686,7 @@ static const struct option longopts[] = {
 	{ "nooutput", 0, 0, 'n' },
 	{ "json", 0, 0, 'j' },
 	{ "reset", 0, 0, 'r' },
+	{ "pretty", 0, 0, 'p' },
 	{ "noupdate", 0, 0, 's' },
 	{ "interval", 1, 0, 't' },
 	{ "version", 0, 0, 'V' },
@@ -678,9 +702,9 @@ int main(int argc, char *argv[])
 	int ch;
 	int fd;
 
-	while ((ch = getopt_long(argc, argv, "hjvVzrnasd:t:e",
+	while ((ch = getopt_long(argc, argv, "hjpvVzrnasd:t:e",
 			longopts, NULL)) != EOF) {
-		switch(ch) {
+		switch (ch) {
 		case 'z':
 			dump_zeros = 1;
 			break;
@@ -702,6 +726,9 @@ int main(int argc, char *argv[])
 		case 'j':
 			json_output = 1;
 			break;
+		case 'p':
+			pretty = 1;
+			break;
 		case 'd':
 			scan_interval = atoi(optarg) * 1000;
 			if (scan_interval <= 0) {
@@ -743,7 +770,7 @@ int main(int argc, char *argv[])
 			perror("ifstat: socket");
 			exit(-1);
 		}
-		if (bind(fd, (struct sockaddr*)&sun, 2+1+strlen(sun.sun_path+1)) < 0) {
+		if (bind(fd, (struct sockaddr *)&sun, 2+1+strlen(sun.sun_path+1)) < 0) {
 			perror("ifstat: bind");
 			exit(-1);
 		}
@@ -801,6 +828,7 @@ int main(int argc, char *argv[])
 		if (!ignore_history) {
 			FILE *tfp;
 			long uptime = -1;
+
 			if ((tfp = fopen("/proc/uptime", "r")) != NULL) {
 				if (fscanf(tfp, "%ld", &uptime) != 1)
 					uptime = -1;
@@ -808,7 +836,8 @@ int main(int argc, char *argv[])
 			}
 			if (uptime >= 0 && time(NULL) >= stb.st_mtime+uptime) {
 				fprintf(stderr, "ifstat: history is aged out, resetting\n");
-				ftruncate(fileno(hist_fp), 0);
+				if (ftruncate(fileno(hist_fp), 0))
+					perror("ifstat: ftruncate");
 			}
 		}
 
@@ -819,11 +848,12 @@ int main(int argc, char *argv[])
 	}
 
 	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) >= 0 &&
-	    (connect(fd, (struct sockaddr*)&sun, 2+1+strlen(sun.sun_path+1)) == 0
+	    (connect(fd, (struct sockaddr *)&sun, 2+1+strlen(sun.sun_path+1)) == 0
 	     || (strcpy(sun.sun_path+1, "ifstat0"),
-		 connect(fd, (struct sockaddr*)&sun, 2+1+strlen(sun.sun_path+1)) == 0))
+		 connect(fd, (struct sockaddr *)&sun, 2+1+strlen(sun.sun_path+1)) == 0))
 	    && verify_forging(fd) == 0) {
 		FILE *sfp = fdopen(fd, "r");
+
 		load_raw_table(sfp);
 		if (hist_db && source_mismatch) {
 			fprintf(stderr, "ifstat: history is stale, ignoring it.\n");
@@ -851,7 +881,8 @@ int main(int argc, char *argv[])
 	}
 
 	if (!no_update) {
-		ftruncate(fileno(hist_fp), 0);
+		if (ftruncate(fileno(hist_fp), 0))
+			perror("ifstat: ftruncate");
 		rewind(hist_fp);
 
 		json_output = 0;
